from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="SummaryAnswer")


@_attrs_define
class SummaryAnswer:
    """Responsible for housing the aggregation for a specific answer.

    Attributes:
        answer (str): The answer selected.
        answer_id (Union[Unset, str]): The answer ID.
        count (Union[Unset, int]): The count of how many times this answer was used in a response.
    """

    answer: str
    answer_id: Union[Unset, str] = UNSET
    count: Union[Unset, int] = 0
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        answer = self.answer
        answer_id = self.answer_id
        count = self.count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "answer": answer,
            }
        )
        if answer_id is not UNSET:
            field_dict["answer_id"] = answer_id
        if count is not UNSET:
            field_dict["count"] = count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        answer = d.pop("answer")

        answer_id = d.pop("answer_id", UNSET)

        count = d.pop("count", UNSET)

        summary_answer = cls(
            answer=answer,
            answer_id=answer_id,
            count=count,
        )

        summary_answer.additional_properties = d
        return summary_answer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .add_to_participant_group import AddToParticipantGroup
from .add_to_participant_group_action import AddToParticipantGroupAction
from .answer_option import AnswerOption
from .attribute import Attribute
from .attribute_value_type_0 import AttributeValueType0
from .attribute_value_type_1_item_type_1 import AttributeValueType1ItemType1
from .automatically_approve import AutomaticallyApprove
from .automatically_approve_action import AutomaticallyApproveAction
from .base_study import BaseStudy
from .base_study_completion_codes_item import BaseStudyCompletionCodesItem
from .base_study_completion_codes_item_code_type import (
    BaseStudyCompletionCodesItemCodeType,
)
from .base_study_completion_option import BaseStudyCompletionOption
from .base_study_content_warnings_item import BaseStudyContentWarningsItem
from .base_study_device_compatibility_item import BaseStudyDeviceCompatibilityItem
from .base_study_peripheral_requirements_item import BaseStudyPeripheralRequirementsItem
from .base_study_prolific_id_option import BaseStudyProlificIdOption
from .base_study_study_labels_item import BaseStudyStudyLabelsItem
from .base_study_submissions_config import BaseStudySubmissionsConfig
from .bulk_bonus import BulkBonus
from .clone_filter_set_json_body import CloneFilterSetJsonBody
from .clone_filter_set_response_201 import CloneFilterSetResponse201
from .create_bonus_payments_json_body import CreateBonusPaymentsJsonBody
from .create_filter_set import CreateFilterSet
from .create_filter_set_response_201 import CreateFilterSetResponse201
from .create_invitation_request import CreateInvitationRequest
from .create_invitation_request_role import CreateInvitationRequestRole
from .create_invitation_response import CreateInvitationResponse
from .create_participant_group_json_body import CreateParticipantGroupJsonBody
from .create_project import CreateProject
from .create_secret import CreateSecret
from .create_study import CreateStudy
from .create_workspace import CreateWorkspace
from .duplicate_study_json_body import DuplicateStudyJsonBody
from .event_type import EventType
from .event_type_list import EventTypeList
from .export_study_method import ExportStudyMethod
from .filter_list import FilterList
from .filter_list_attributes import FilterListAttributes
from .filter_list_attributes_type import FilterListAttributesType
from .filter_list_detailed_attributes import FilterListDetailedAttributes
from .filter_list_links import FilterListLinks
from .filter_list_meta import FilterListMeta
from .filter_set import FilterSet
from .filter_set_list import FilterSetList
from .filter_set_participant_count import FilterSetParticipantCount
from .get_filter_set_response_200 import GetFilterSetResponse200
from .get_participant_groups_active import GetParticipantGroupsActive
from .get_studies_state import GetStudiesState
from .invitation import Invitation
from .invitation_invitee import InvitationInvitee
from .invitation_status import InvitationStatus
from .lock_filter_set_response_200 import LockFilterSetResponse200
from .manually_review import ManuallyReview
from .manually_review_action import ManuallyReviewAction
from .message import Message
from .message_data import MessageData
from .message_data_category import MessageDataCategory
from .messages import Messages
from .mutually_exclusive_study_collection_update import (
    MutuallyExclusiveStudyCollectionUpdate,
)
from .mutually_exclusive_study_collections_response import (
    MutuallyExclusiveStudyCollectionsResponse,
)
from .participant_group import ParticipantGroup
from .participant_group_feeder_studies_item import ParticipantGroupFeederStudiesItem
from .participant_group_feeder_studies_item_feeder_completion_codes_item import (
    ParticipantGroupFeederStudiesItemFeederCompletionCodesItem,
)
from .participant_group_feeder_studies_item_feeder_completion_codes_item_action import (
    ParticipantGroupFeederStudiesItemFeederCompletionCodesItemAction,
)
from .participant_group_list_response import ParticipantGroupListResponse
from .participant_group_membership import ParticipantGroupMembership
from .participant_group_membership_list_response import (
    ParticipantGroupMembershipListResponse,
)
from .participant_group_update import ParticipantGroupUpdate
from .participant_i_ds import ParticipantIDs
from .participant_id_list import ParticipantIDList
from .project import Project
from .project_id import ProjectID
from .project_short import ProjectShort
from .project_short_list_response import ProjectShortListResponse
from .question import Question
from .question_response import QuestionResponse
from .question_type import QuestionType
from .range_filter import RangeFilter
from .range_filter_list_attributes import RangeFilterListAttributes
from .range_filter_list_attributes_data_type import RangeFilterListAttributesDataType
from .range_filter_list_detailed_response import RangeFilterListDetailedResponse
from .range_filter_list_response import RangeFilterListResponse
from .range_filter_selected_range import RangeFilterSelectedRange
from .remove_from_participant_group import RemoveFromParticipantGroup
from .remove_from_participant_group_action import RemoveFromParticipantGroupAction
from .request_return import RequestReturn
from .request_return_action import RequestReturnAction
from .request_submission_return_json_body import RequestSubmissionReturnJsonBody
from .requirement import Requirement
from .requirement_query import RequirementQuery
from .requirements_count import RequirementsCount
from .requirements_count_request import RequirementsCountRequest
from .requirements_response import RequirementsResponse
from .response_answer import ResponseAnswer
from .response_in import ResponseIn
from .response_out import ResponseOut
from .return_requested_response import ReturnRequestedResponse
from .return_requested_response_status import ReturnRequestedResponseStatus
from .secret_detail import SecretDetail
from .secret_list import SecretList
from .section import Section
from .select_filter import SelectFilter
from .select_filter_list_attributes import SelectFilterListAttributes
from .select_filter_list_attributes_choices import SelectFilterListAttributesChoices
from .select_filter_list_attributes_data_type import SelectFilterListAttributesDataType
from .select_filter_list_detailed_response import SelectFilterListDetailedResponse
from .select_filter_list_response import SelectFilterListResponse
from .select_filter_weightings import SelectFilterWeightings
from .send_bulk_message import SendBulkMessage
from .send_message import SendMessage
from .studies_list_response import StudiesListResponse
from .study import Study
from .study_cost_request import StudyCostRequest
from .study_cost_response import StudyCostResponse
from .study_short import StudyShort
from .study_short_status import StudyShortStatus
from .study_short_study_type import StudyShortStudyType
from .study_status import StudyStatus
from .study_transition import StudyTransition
from .submission import Submission
from .submission_detail import SubmissionDetail
from .submission_detail_status import SubmissionDetailStatus
from .submission_i_ds import SubmissionIDs
from .submission_list_response import SubmissionListResponse
from .submission_short import SubmissionShort
from .submission_short_status import SubmissionShortStatus
from .submission_status import SubmissionStatus
from .submission_transition import SubmissionTransition
from .submission_transition_action import SubmissionTransitionAction
from .submission_transition_rejection_category import (
    SubmissionTransitionRejectionCategory,
)
from .subscription_detail import SubscriptionDetail
from .subscription_event import SubscriptionEvent
from .subscription_event_list import SubscriptionEventList
from .subscription_event_payload import SubscriptionEventPayload
from .subscription_event_status import SubscriptionEventStatus
from .subscription_list import SubscriptionList
from .subscription_update_detail import SubscriptionUpdateDetail
from .summary import Summary
from .summary_answer import SummaryAnswer
from .summary_question import SummaryQuestion
from .transition_mutually_exclusive_study_collection_json_body import (
    TransitionMutuallyExclusiveStudyCollectionJsonBody,
)
from .transition_mutually_exclusive_study_collection_json_body_action import (
    TransitionMutuallyExclusiveStudyCollectionJsonBodyAction,
)
from .unlock_filter_set_response_200 import UnlockFilterSetResponse200
from .update_filter_set import UpdateFilterSet
from .update_filter_set_response_200 import UpdateFilterSetResponse200
from .user import User
from .workspace import Workspace
from .workspace_balance import WorkspaceBalance
from .workspace_balance_available_balance_breakdown import (
    WorkspaceBalanceAvailableBalanceBreakdown,
)
from .workspace_balance_balance_breakdown import WorkspaceBalanceBalanceBreakdown
from .workspace_id import WorkspaceID
from .workspace_short import WorkspaceShort
from .workspace_user import WorkspaceUser
from .workspaces_list_response import WorkspacesListResponse

__all__ = (
    "AddToParticipantGroup",
    "AddToParticipantGroupAction",
    "AnswerOption",
    "Attribute",
    "AttributeValueType0",
    "AttributeValueType1ItemType1",
    "AutomaticallyApprove",
    "AutomaticallyApproveAction",
    "BaseStudy",
    "BaseStudyCompletionCodesItem",
    "BaseStudyCompletionCodesItemCodeType",
    "BaseStudyCompletionOption",
    "BaseStudyContentWarningsItem",
    "BaseStudyDeviceCompatibilityItem",
    "BaseStudyPeripheralRequirementsItem",
    "BaseStudyProlificIdOption",
    "BaseStudyStudyLabelsItem",
    "BaseStudySubmissionsConfig",
    "BulkBonus",
    "CloneFilterSetJsonBody",
    "CloneFilterSetResponse201",
    "CreateBonusPaymentsJsonBody",
    "CreateFilterSet",
    "CreateFilterSetResponse201",
    "CreateInvitationRequest",
    "CreateInvitationRequestRole",
    "CreateInvitationResponse",
    "CreateParticipantGroupJsonBody",
    "CreateProject",
    "CreateSecret",
    "CreateStudy",
    "CreateWorkspace",
    "DuplicateStudyJsonBody",
    "EventType",
    "EventTypeList",
    "ExportStudyMethod",
    "FilterList",
    "FilterListAttributes",
    "FilterListAttributesType",
    "FilterListDetailedAttributes",
    "FilterListLinks",
    "FilterListMeta",
    "FilterSet",
    "FilterSetList",
    "FilterSetParticipantCount",
    "GetFilterSetResponse200",
    "GetParticipantGroupsActive",
    "GetStudiesState",
    "Invitation",
    "InvitationInvitee",
    "InvitationStatus",
    "LockFilterSetResponse200",
    "ManuallyReview",
    "ManuallyReviewAction",
    "Message",
    "MessageData",
    "MessageDataCategory",
    "Messages",
    "MutuallyExclusiveStudyCollectionsResponse",
    "MutuallyExclusiveStudyCollectionUpdate",
    "ParticipantGroup",
    "ParticipantGroupFeederStudiesItem",
    "ParticipantGroupFeederStudiesItemFeederCompletionCodesItem",
    "ParticipantGroupFeederStudiesItemFeederCompletionCodesItemAction",
    "ParticipantGroupListResponse",
    "ParticipantGroupMembership",
    "ParticipantGroupMembershipListResponse",
    "ParticipantGroupUpdate",
    "ParticipantIDList",
    "ParticipantIDs",
    "Project",
    "ProjectID",
    "ProjectShort",
    "ProjectShortListResponse",
    "Question",
    "QuestionResponse",
    "QuestionType",
    "RangeFilter",
    "RangeFilterListAttributes",
    "RangeFilterListAttributesDataType",
    "RangeFilterListDetailedResponse",
    "RangeFilterListResponse",
    "RangeFilterSelectedRange",
    "RemoveFromParticipantGroup",
    "RemoveFromParticipantGroupAction",
    "RequestReturn",
    "RequestReturnAction",
    "RequestSubmissionReturnJsonBody",
    "Requirement",
    "RequirementQuery",
    "RequirementsCount",
    "RequirementsCountRequest",
    "RequirementsResponse",
    "ResponseAnswer",
    "ResponseIn",
    "ResponseOut",
    "ReturnRequestedResponse",
    "ReturnRequestedResponseStatus",
    "SecretDetail",
    "SecretList",
    "Section",
    "SelectFilter",
    "SelectFilterListAttributes",
    "SelectFilterListAttributesChoices",
    "SelectFilterListAttributesDataType",
    "SelectFilterListDetailedResponse",
    "SelectFilterListResponse",
    "SelectFilterWeightings",
    "SendBulkMessage",
    "SendMessage",
    "StudiesListResponse",
    "Study",
    "StudyCostRequest",
    "StudyCostResponse",
    "StudyShort",
    "StudyShortStatus",
    "StudyShortStudyType",
    "StudyStatus",
    "StudyTransition",
    "Submission",
    "SubmissionDetail",
    "SubmissionDetailStatus",
    "SubmissionIDs",
    "SubmissionListResponse",
    "SubmissionShort",
    "SubmissionShortStatus",
    "SubmissionStatus",
    "SubmissionTransition",
    "SubmissionTransitionAction",
    "SubmissionTransitionRejectionCategory",
    "SubscriptionDetail",
    "SubscriptionEvent",
    "SubscriptionEventList",
    "SubscriptionEventPayload",
    "SubscriptionEventStatus",
    "SubscriptionList",
    "SubscriptionUpdateDetail",
    "Summary",
    "SummaryAnswer",
    "SummaryQuestion",
    "TransitionMutuallyExclusiveStudyCollectionJsonBody",
    "TransitionMutuallyExclusiveStudyCollectionJsonBodyAction",
    "UnlockFilterSetResponse200",
    "UpdateFilterSet",
    "UpdateFilterSetResponse200",
    "User",
    "Workspace",
    "WorkspaceBalance",
    "WorkspaceBalanceAvailableBalanceBreakdown",
    "WorkspaceBalanceBalanceBreakdown",
    "WorkspaceID",
    "WorkspaceShort",
    "WorkspacesListResponse",
    "WorkspaceUser",
)



from enum import Enum


class RemoveFromParticipantGroupAction(str, Enum):
    REMOVE_FROM_PARTICIPANT_GROUP = "REMOVE_FROM_PARTICIPANT_GROUP"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.base_study_prolific_id_option import BaseStudyProlificIdOption
from typing import Union
from ..models.base_study_content_warnings_item import BaseStudyContentWarningsItem
from ..models.base_study_study_labels_item import BaseStudyStudyLabelsItem
from ..models.study_status import StudyStatus
from ..models.base_study_completion_option import BaseStudyCompletionOption
from ..models.base_study_peripheral_requirements_item import (
    BaseStudyPeripheralRequirementsItem,
)
from typing import Union
from typing import List
from ..types import UNSET, Unset
from ..models.base_study_device_compatibility_item import (
    BaseStudyDeviceCompatibilityItem,
)
from typing import Dict

if TYPE_CHECKING:
    from ..models.base_study_completion_codes_item import BaseStudyCompletionCodesItem
    from ..models.range_filter import RangeFilter
    from ..models.base_study_submissions_config import BaseStudySubmissionsConfig
    from ..models.select_filter import SelectFilter


T = TypeVar("T", bound="Study")


@_attrs_define
class Study:
    r"""
    Example:
        {'id': '60d9aadeb86739de712faee0', 'name': "Study about API's", 'internal_name': "WIT-2021 Study about API's
            version 2", 'description': 'This study aims to determine how to make a good public API', 'external_study_url':
            'https://eggs-experriment.com?participant={{%PROLIFIC_PID%}}', 'prolific_id_option': 'url_parameters',
            'completion_option': 'url', 'completion_codes': [{'code': 'ABC123', 'code_type': 'COMPLETED', 'actions':
            [{'action': 'AUTOMATICALLY_APPROVE'}]}, {'code': 'DEF234', 'code_type': 'FOLLOW_UP_STUDY', 'actions':
            [{'action': 'AUTOMATICALLY_APPROVE'}, {'action': 'ADD_TO_PARTICIPANT_GROUP', 'participant_group':
            '619e049f7648a4e1f8f3645b'}]}], 'total_available_places': 30, 'estimated_completion_time': 5,
            'maximum_allowed_time': 25, 'reward': 100, 'device_compatibility': ['desktop'], 'peripheral_requirements': [],
            'filters': [], 'filter_set_id': None, 'filter_set_version': None, 'status': 'UNPUBLISHED', 'study_labels':
            ['interview'], 'content_warnings': ['sensitive']}

    Attributes:
        id (str): Study id. It is created by Prolific. **Read only**.
        status (StudyStatus): Status of the study. **Read only**.

            To change the status you can use `/api/v1/studies/{id}/transition/`
        name (Union[Unset, str]): Public name or title of the study
        internal_name (Union[Unset, None, str]): Internal name of the study, not shown to participants
        description (Union[Unset, str]): Description of the study for the participants to read before
            starting the study
        external_study_url (Union[Unset, str]): URL of the survey or experiment you want participant to access. You can
            pass URL search parameters to your survey or experiment

            * Participant id {{%PROLIFIC_PID%}}
            * Study id {{%STUDY_ID%}}
            * Session id {{%SESSION_ID%}}

            For example `https://eggs-experriment.com?participant={{%PROLIFIC_PID%}}`
        prolific_id_option (Union[Unset, BaseStudyProlificIdOption]): Use 'question' if you will add a question in your
            survey or experiment asking the participant ID

            **Recommended** Use 'url_parameters' if your survey or experiment can retrieve and store those parameters for
            your analysis.

            Use 'not_required' if you don't need to record them
        completion_option (Union[Unset, BaseStudyCompletionOption]): Use 'url' if you will redirect the user back to
            prolific using a url,
            the url you will use in your experiment
            or survey to go back is https://app.prolific.com/submissions/complete?cc={code}

            Use 'code' when the participants will manually input the code, at the
            end of the experiment you will tell the participants the code. Note that
            the {code} you have to give is one of the completion codes you define below in the `completion_codes` argument.
        completion_code (Union[Unset, str]): This field has been added by pyrolific to work around issues with the
            completion code api.
            https://github.com/rwblair/pyrolific/issues/3
        completion_code_action (Union[Unset, str]): This field has been added by pyrolific to work around issues with
            the completion code api.
            https://github.com/rwblair/pyrolific/issues/3
        completion_codes (Union[Unset, List['BaseStudyCompletionCodesItem']]): Specify at least one completion code for
            your study. A participant will enter one of these codes when they complete your study.

            Each code must be unique within a study.

            You can specify as many actions as you like per code.
        total_available_places (Union[Unset, float]): How many participants are you looking to recruit
        estimated_completion_time (Union[Unset, float]): Estimated duration in minutes of the experiment or survey
        maximum_allowed_time (Union[Unset, float]): Max time in minutes for a participant to finish the submission.
            Submissions are timed out if it takes longer.

            If it is not provided the default value is set to the max value.

            The min value is calculated as two minutes plus two times the estimated time plus two times the square root of
            the estimated time
        reward (Union[Unset, float]): How much are you going to pay the participants in cents. We
            use the currency of your account.
        device_compatibility (Union[Unset, List[BaseStudyDeviceCompatibilityItem]]): Add all devices that participants
            can use.
            You can include one or more options.

            An empty array indicates that all options are available.
        peripheral_requirements (Union[Unset, List[BaseStudyPeripheralRequirementsItem]]): Add all requirements that
            participants have to meet.

            An empty array indicates that there are no extra peripheral requirements.
        filters (Union[Unset, None, List[Union['RangeFilter', 'SelectFilter']]]): Array of filters.

            Use empty array for "Everyone"
        filter_set_id (Union[Unset, None, str]): The ID of a filter set, from which filters for the study will be taken.

            Note, this cannot be used in combination with additional filters via the `filters` field.
        filter_set_version (Union[Unset, None, int]): The version of the filter set to be used.

            If not provided, this will default to the latest available version at the time of applying the filter set.
        naivety_distribution_rate (Union[Unset, None, float]): Control the balance between speed of your studies and the
            naivety of the participants.

            If not defined, by default Prolific calculates the best rate for most studies
            taking into account the `filters` and the `total_available_places` needed for this study.

            Use 0 if your priority is speed. When this property is set to 0 all eligible participants will have access
            to your study at the same time, without any prioritization.

            You can also set this at a workspace and project level.
        project (Union[Unset, str]): Project id, this is optional and if not supplied with be the put in the default
            workspace and project.
        submissions_config (Union[Unset, BaseStudySubmissionsConfig]): **BETA**: This is a beta feature and is currently
            only available to selected workspaces.
            It is being tested and evaluated for effectiveness and user experience before being released to all users.

            **Advanced**: This helps with faster data collection. Your survey system will need to handle providing a
            unique experience each time the participant takes the study.

            Configuration related to study submissions. The purpose of this field is to capture any configuration options
            that impact the submissions made by participants in a study.
        study_labels (Union[Unset, List[BaseStudyStudyLabelsItem]]): This field allows you to tag studies with
            information about the type/topic of the study and the kind of work involved in completing it.

            We plan to make this information available to participants for easier self-selection. At present these options
            are mutually exclusive and only a single option can be selected, however in the future available categories will
            expand.
        content_warnings (Union[Unset, List[BaseStudyContentWarningsItem]]): Allow researchers to define content
            warnings for their study.

            At present these options are mutually exclusive and only a single option can be selected, however in the future
            available warnings will expand.
        metadata (Union[Unset, None, str]): This field can be used to store extra information required for a system
            integration.
            For example, it could be some JSON, XML, an integer, or a string.

            Examples could include:

              - `123345` - An ID from your system, that helps with linkage when returning the study.
              - `{ \"id\": \"45\", \"type\": \"finance\"}` - Some JSON that you want to store.
    """

    id: str
    status: StudyStatus
    name: Union[Unset, str] = UNSET
    internal_name: Union[Unset, None, str] = UNSET
    description: Union[Unset, str] = UNSET
    external_study_url: Union[Unset, str] = UNSET
    prolific_id_option: Union[Unset, BaseStudyProlificIdOption] = UNSET
    completion_option: Union[Unset, BaseStudyCompletionOption] = UNSET
    completion_code: Union[Unset, str] = UNSET
    completion_code_action: Union[Unset, str] = UNSET
    completion_codes: Union[Unset, List["BaseStudyCompletionCodesItem"]] = UNSET
    total_available_places: Union[Unset, float] = UNSET
    estimated_completion_time: Union[Unset, float] = UNSET
    maximum_allowed_time: Union[Unset, float] = UNSET
    reward: Union[Unset, float] = UNSET
    device_compatibility: Union[Unset, List[BaseStudyDeviceCompatibilityItem]] = UNSET
    peripheral_requirements: Union[
        Unset, List[BaseStudyPeripheralRequirementsItem]
    ] = UNSET
    filters: Union[Unset, None, List[Union["RangeFilter", "SelectFilter"]]] = UNSET
    filter_set_id: Union[Unset, None, str] = UNSET
    filter_set_version: Union[Unset, None, int] = UNSET
    naivety_distribution_rate: Union[Unset, None, float] = UNSET
    project: Union[Unset, str] = UNSET
    submissions_config: Union[Unset, "BaseStudySubmissionsConfig"] = UNSET
    study_labels: Union[Unset, List[BaseStudyStudyLabelsItem]] = UNSET
    content_warnings: Union[Unset, List[BaseStudyContentWarningsItem]] = UNSET
    metadata: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.select_filter import SelectFilter

        id = self.id
        status = self.status.value

        name = self.name
        internal_name = self.internal_name
        description = self.description
        external_study_url = self.external_study_url
        prolific_id_option: Union[Unset, str] = UNSET
        if not isinstance(self.prolific_id_option, Unset):
            prolific_id_option = self.prolific_id_option.value

        completion_option: Union[Unset, str] = UNSET
        if not isinstance(self.completion_option, Unset):
            completion_option = self.completion_option.value

        completion_code = self.completion_code
        completion_code_action = self.completion_code_action
        completion_codes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.completion_codes, Unset):
            completion_codes = []
            for completion_codes_item_data in self.completion_codes:
                completion_codes_item = completion_codes_item_data.to_dict()

                completion_codes.append(completion_codes_item)

        total_available_places = self.total_available_places
        estimated_completion_time = self.estimated_completion_time
        maximum_allowed_time = self.maximum_allowed_time
        reward = self.reward
        device_compatibility: Union[Unset, List[str]] = UNSET
        if not isinstance(self.device_compatibility, Unset):
            device_compatibility = []
            for device_compatibility_item_data in self.device_compatibility:
                device_compatibility_item = device_compatibility_item_data.value

                device_compatibility.append(device_compatibility_item)

        peripheral_requirements: Union[Unset, List[str]] = UNSET
        if not isinstance(self.peripheral_requirements, Unset):
            peripheral_requirements = []
            for peripheral_requirements_item_data in self.peripheral_requirements:
                peripheral_requirements_item = peripheral_requirements_item_data.value

                peripheral_requirements.append(peripheral_requirements_item)

        filters: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            if self.filters is None:
                filters = None
            else:
                filters = []
                for filters_item_data in self.filters:
                    filters_item: Dict[str, Any]

                    if isinstance(filters_item_data, SelectFilter):
                        filters_item = filters_item_data.to_dict()

                    else:
                        filters_item = filters_item_data.to_dict()

                    filters.append(filters_item)

        filter_set_id = self.filter_set_id
        filter_set_version = self.filter_set_version
        naivety_distribution_rate = self.naivety_distribution_rate
        project = self.project
        submissions_config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.submissions_config, Unset):
            submissions_config = self.submissions_config.to_dict()

        study_labels: Union[Unset, List[str]] = UNSET
        if not isinstance(self.study_labels, Unset):
            study_labels = []
            for study_labels_item_data in self.study_labels:
                study_labels_item = study_labels_item_data.value

                study_labels.append(study_labels_item)

        content_warnings: Union[Unset, List[str]] = UNSET
        if not isinstance(self.content_warnings, Unset):
            content_warnings = []
            for content_warnings_item_data in self.content_warnings:
                content_warnings_item = content_warnings_item_data.value

                content_warnings.append(content_warnings_item)

        metadata = self.metadata

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "status": status,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if internal_name is not UNSET:
            field_dict["internal_name"] = internal_name
        if description is not UNSET:
            field_dict["description"] = description
        if external_study_url is not UNSET:
            field_dict["external_study_url"] = external_study_url
        if prolific_id_option is not UNSET:
            field_dict["prolific_id_option"] = prolific_id_option
        if completion_option is not UNSET:
            field_dict["completion_option"] = completion_option
        if completion_code is not UNSET:
            field_dict["completion_code"] = completion_code
        if completion_code_action is not UNSET:
            field_dict["completion_code_action"] = completion_code_action
        if completion_codes is not UNSET:
            field_dict["completion_codes"] = completion_codes
        if total_available_places is not UNSET:
            field_dict["total_available_places"] = total_available_places
        if estimated_completion_time is not UNSET:
            field_dict["estimated_completion_time"] = estimated_completion_time
        if maximum_allowed_time is not UNSET:
            field_dict["maximum_allowed_time"] = maximum_allowed_time
        if reward is not UNSET:
            field_dict["reward"] = reward
        if device_compatibility is not UNSET:
            field_dict["device_compatibility"] = device_compatibility
        if peripheral_requirements is not UNSET:
            field_dict["peripheral_requirements"] = peripheral_requirements
        if filters is not UNSET:
            field_dict["filters"] = filters
        if filter_set_id is not UNSET:
            field_dict["filter_set_id"] = filter_set_id
        if filter_set_version is not UNSET:
            field_dict["filter_set_version"] = filter_set_version
        if naivety_distribution_rate is not UNSET:
            field_dict["naivety_distribution_rate"] = naivety_distribution_rate
        if project is not UNSET:
            field_dict["project"] = project
        if submissions_config is not UNSET:
            field_dict["submissions_config"] = submissions_config
        if study_labels is not UNSET:
            field_dict["study_labels"] = study_labels
        if content_warnings is not UNSET:
            field_dict["content_warnings"] = content_warnings
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_study_completion_codes_item import (
            BaseStudyCompletionCodesItem,
        )
        from ..models.range_filter import RangeFilter
        from ..models.base_study_submissions_config import BaseStudySubmissionsConfig
        from ..models.select_filter import SelectFilter

        d = src_dict.copy()
        id = d.pop("id")

        status = StudyStatus(d.pop("status"))

        name = d.pop("name", UNSET)

        internal_name = d.pop("internal_name", UNSET)

        description = d.pop("description", UNSET)

        external_study_url = d.pop("external_study_url", UNSET)

        _prolific_id_option = d.pop("prolific_id_option", UNSET)
        prolific_id_option: Union[Unset, BaseStudyProlificIdOption]
        if isinstance(_prolific_id_option, Unset):
            prolific_id_option = UNSET
        else:
            prolific_id_option = BaseStudyProlificIdOption(_prolific_id_option)

        _completion_option = d.pop("completion_option", UNSET)
        completion_option: Union[Unset, BaseStudyCompletionOption]
        if isinstance(_completion_option, Unset):
            completion_option = UNSET
        else:
            completion_option = BaseStudyCompletionOption(_completion_option)

        completion_code = d.pop("completion_code", UNSET)

        completion_code_action = d.pop("completion_code_action", UNSET)

        completion_codes = []
        _completion_codes = d.pop("completion_codes", UNSET)
        for completion_codes_item_data in _completion_codes or []:
            completion_codes_item = BaseStudyCompletionCodesItem.from_dict(
                completion_codes_item_data
            )

            completion_codes.append(completion_codes_item)

        total_available_places = d.pop("total_available_places", UNSET)

        estimated_completion_time = d.pop("estimated_completion_time", UNSET)

        maximum_allowed_time = d.pop("maximum_allowed_time", UNSET)

        reward = d.pop("reward", UNSET)

        device_compatibility = []
        _device_compatibility = d.pop("device_compatibility", UNSET)
        for device_compatibility_item_data in _device_compatibility or []:
            device_compatibility_item = BaseStudyDeviceCompatibilityItem(
                device_compatibility_item_data
            )

            device_compatibility.append(device_compatibility_item)

        peripheral_requirements = []
        _peripheral_requirements = d.pop("peripheral_requirements", UNSET)
        for peripheral_requirements_item_data in _peripheral_requirements or []:
            peripheral_requirements_item = BaseStudyPeripheralRequirementsItem(
                peripheral_requirements_item_data
            )

            peripheral_requirements.append(peripheral_requirements_item)

        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(
                data: object,
            ) -> Union["RangeFilter", "SelectFilter"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_0 = SelectFilter.from_dict(data)

                    return filters_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                filters_item_type_1 = RangeFilter.from_dict(data)

                return filters_item_type_1

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        filter_set_id = d.pop("filter_set_id", UNSET)

        filter_set_version = d.pop("filter_set_version", UNSET)

        naivety_distribution_rate = d.pop("naivety_distribution_rate", UNSET)

        project = d.pop("project", UNSET)

        _submissions_config = d.pop("submissions_config", UNSET)
        submissions_config: Union[Unset, BaseStudySubmissionsConfig]
        if isinstance(_submissions_config, Unset):
            submissions_config = UNSET
        else:
            submissions_config = BaseStudySubmissionsConfig.from_dict(
                _submissions_config
            )

        study_labels = []
        _study_labels = d.pop("study_labels", UNSET)
        for study_labels_item_data in _study_labels or []:
            study_labels_item = BaseStudyStudyLabelsItem(study_labels_item_data)

            study_labels.append(study_labels_item)

        content_warnings = []
        _content_warnings = d.pop("content_warnings", UNSET)
        for content_warnings_item_data in _content_warnings or []:
            content_warnings_item = BaseStudyContentWarningsItem(
                content_warnings_item_data
            )

            content_warnings.append(content_warnings_item)

        metadata = d.pop("metadata", UNSET)

        study = cls(
            id=id,
            status=status,
            name=name,
            internal_name=internal_name,
            description=description,
            external_study_url=external_study_url,
            prolific_id_option=prolific_id_option,
            completion_option=completion_option,
            completion_code=completion_code,
            completion_code_action=completion_code_action,
            completion_codes=completion_codes,
            total_available_places=total_available_places,
            estimated_completion_time=estimated_completion_time,
            maximum_allowed_time=maximum_allowed_time,
            reward=reward,
            device_compatibility=device_compatibility,
            peripheral_requirements=peripheral_requirements,
            filters=filters,
            filter_set_id=filter_set_id,
            filter_set_version=filter_set_version,
            naivety_distribution_rate=naivety_distribution_rate,
            project=project,
            submissions_config=submissions_config,
            study_labels=study_labels,
            content_warnings=content_warnings,
            metadata=metadata,
        )

        study.additional_properties = d
        return study

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import List
from typing import Dict

if TYPE_CHECKING:
    from ..models.response_answer import ResponseAnswer


T = TypeVar("T", bound="QuestionResponse")


@_attrs_define
class QuestionResponse:
    """Responsible for defining an answer to a survey question

    Attributes:
        answers (List['ResponseAnswer']): The answers selected.
        question_id (str): The question ID.
        question_title (str): The title of the survey question.
    """

    answers: List["ResponseAnswer"]
    question_id: str
    question_title: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        answers = []
        for answers_item_data in self.answers:
            answers_item = answers_item_data.to_dict()

            answers.append(answers_item)

        question_id = self.question_id
        question_title = self.question_title

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "answers": answers,
                "question_id": question_id,
                "question_title": question_title,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.response_answer import ResponseAnswer

        d = src_dict.copy()
        answers = []
        _answers = d.pop("answers")
        for answers_item_data in _answers:
            answers_item = ResponseAnswer.from_dict(answers_item_data)

            answers.append(answers_item)

        question_id = d.pop("question_id")

        question_title = d.pop("question_title")

        question_response = cls(
            answers=answers,
            question_id=question_id,
            question_title=question_title,
        )

        question_response.additional_properties = d
        return question_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="SubscriptionUpdateDetail")


@_attrs_define
class SubscriptionUpdateDetail:
    """
    Attributes:
        event_type (str): The name of the event type associated to the subscription.
        target_url (str): The URL that the subscription will notify when your event type is triggered.
        is_enabled (Union[Unset, bool]): Whether the subscription is enabled or not.
    """

    event_type: str
    target_url: str
    is_enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        event_type = self.event_type
        target_url = self.target_url
        is_enabled = self.is_enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "event_type": event_type,
                "target_url": target_url,
            }
        )
        if is_enabled is not UNSET:
            field_dict["is_enabled"] = is_enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        event_type = d.pop("event_type")

        target_url = d.pop("target_url")

        is_enabled = d.pop("is_enabled", UNSET)

        subscription_update_detail = cls(
            event_type=event_type,
            target_url=target_url,
            is_enabled=is_enabled,
        )

        subscription_update_detail.additional_properties = d
        return subscription_update_detail

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast, List
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="WorkspaceUser")


@_attrs_define
class WorkspaceUser:
    """
    Attributes:
        id (str): Id of user
        name (Union[Unset, str]): Name of user
        email (Union[Unset, str]): email of user
        roles (Union[Unset, List[str]]): User roles in workspace
    """

    id: str
    name: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    roles: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        email = self.email
        roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.roles, Unset):
            roles = self.roles

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if email is not UNSET:
            field_dict["email"] = email
        if roles is not UNSET:
            field_dict["roles"] = roles

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name", UNSET)

        email = d.pop("email", UNSET)

        roles = cast(List[str], d.pop("roles", UNSET))

        workspace_user = cls(
            id=id,
            name=name,
            email=email,
            roles=roles,
        )

        workspace_user.additional_properties = d
        return workspace_user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.filter_set import FilterSet


T = TypeVar("T", bound="FilterSetList")


@_attrs_define
class FilterSetList:
    """
    Attributes:
        results (Union[Unset, List['FilterSet']]): List of filter sets.
    """

    results: Union[Unset, List["FilterSet"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        results: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.results, Unset):
            results = []
            for results_item_data in self.results:
                results_item = results_item_data.to_dict()

                results.append(results_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if results is not UNSET:
            field_dict["results"] = results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.filter_set import FilterSet

        d = src_dict.copy()
        results = []
        _results = d.pop("results", UNSET)
        for results_item_data in _results or []:
            results_item = FilterSet.from_dict(results_item_data)

            results.append(results_item)

        filter_set_list = cls(
            results=results,
        )

        filter_set_list.additional_properties = d
        return filter_set_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.submission_transition_rejection_category import (
    SubmissionTransitionRejectionCategory,
)
from typing import Union
from ..models.submission_transition_action import SubmissionTransitionAction
from ..types import UNSET, Unset


T = TypeVar("T", bound="SubmissionTransition")


@_attrs_define
class SubmissionTransition:
    """
    Attributes:
        action (SubmissionTransitionAction): Action to execute. Example: APPROVE.
        message (Union[Unset, str]): Required if action is 'REJECT'. Message sent to the participant
            explaining the reason for the rejection.
            It must be at least 100 chars long. Example: Good explanation of the situation..
        rejection_category (Union[Unset, SubmissionTransitionRejectionCategory]): Required if action is 'REJECT', it
            sums as the category of
            the rejection. Example: LOW_EFFORT.
    """

    action: SubmissionTransitionAction
    message: Union[Unset, str] = UNSET
    rejection_category: Union[Unset, SubmissionTransitionRejectionCategory] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        action = self.action.value

        message = self.message
        rejection_category: Union[Unset, str] = UNSET
        if not isinstance(self.rejection_category, Unset):
            rejection_category = self.rejection_category.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "action": action,
            }
        )
        if message is not UNSET:
            field_dict["message"] = message
        if rejection_category is not UNSET:
            field_dict["rejection_category"] = rejection_category

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        action = SubmissionTransitionAction(d.pop("action"))

        message = d.pop("message", UNSET)

        _rejection_category = d.pop("rejection_category", UNSET)
        rejection_category: Union[Unset, SubmissionTransitionRejectionCategory]
        if isinstance(_rejection_category, Unset):
            rejection_category = UNSET
        else:
            rejection_category = SubmissionTransitionRejectionCategory(
                _rejection_category
            )

        submission_transition = cls(
            action=action,
            message=message,
            rejection_category=rejection_category,
        )

        submission_transition.additional_properties = d
        return submission_transition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union
from typing import cast, List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.select_filter_weightings import SelectFilterWeightings


T = TypeVar("T", bound="SelectFilter")


@_attrs_define
class SelectFilter:
    r"""
    Attributes:
        filter_id (str): ID of the "select" type filter.
        selected_values (List[str]): This schema applies for filters of the `select` type, as defined in the [filter
            list response](\#tag/Filters/paths/~1api~1v1~1filters~1/get).

            Array of IDs matching the response IDs, from the `select` filter's `choices` (see response linked above).

            String format should match the `data_type` of the `select` filter's `choices` (see response linked above).
        weightings (Union[Unset, SelectFilterWeightings]): Ratios to control the distribution of participants across the
            selected values.

            Integer percentages, floats, and exact quantities are valid inputs.
    """

    filter_id: str
    selected_values: List[str]
    weightings: Union[Unset, "SelectFilterWeightings"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        filter_id = self.filter_id
        selected_values = self.selected_values

        weightings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.weightings, Unset):
            weightings = self.weightings.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "filter_id": filter_id,
                "selected_values": selected_values,
            }
        )
        if weightings is not UNSET:
            field_dict["weightings"] = weightings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.select_filter_weightings import SelectFilterWeightings

        d = src_dict.copy()
        filter_id = d.pop("filter_id")

        selected_values = cast(List[str], d.pop("selected_values"))

        _weightings = d.pop("weightings", UNSET)
        weightings: Union[Unset, SelectFilterWeightings]
        if isinstance(_weightings, Unset):
            weightings = UNSET
        else:
            weightings = SelectFilterWeightings.from_dict(_weightings)

        select_filter = cls(
            filter_id=filter_id,
            selected_values=selected_values,
            weightings=weightings,
        )

        select_filter.additional_properties = d
        return select_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.range_filter_list_detailed_response import (
        RangeFilterListDetailedResponse,
    )
    from ..models.select_filter_list_detailed_response import (
        SelectFilterListDetailedResponse,
    )
    from ..models.range_filter_list_response import RangeFilterListResponse
    from ..models.filter_list_meta import FilterListMeta
    from ..models.select_filter_list_response import SelectFilterListResponse
    from ..models.filter_list_links import FilterListLinks


T = TypeVar("T", bound="FilterList")


@_attrs_define
class FilterList:
    """
    Attributes:
        results (Union[Unset, List[Union['RangeFilterListDetailedResponse', 'RangeFilterListResponse',
            'SelectFilterListDetailedResponse', 'SelectFilterListResponse']]]):
        field_links (Union[Unset, FilterListLinks]):
        meta (Union[Unset, FilterListMeta]):
    """

    results: Union[
        Unset,
        List[
            Union[
                "RangeFilterListDetailedResponse",
                "RangeFilterListResponse",
                "SelectFilterListDetailedResponse",
                "SelectFilterListResponse",
            ]
        ],
    ] = UNSET
    field_links: Union[Unset, "FilterListLinks"] = UNSET
    meta: Union[Unset, "FilterListMeta"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.select_filter_list_detailed_response import (
            SelectFilterListDetailedResponse,
        )
        from ..models.range_filter_list_response import RangeFilterListResponse
        from ..models.select_filter_list_response import SelectFilterListResponse

        results: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.results, Unset):
            results = []
            for results_item_data in self.results:
                results_item: Dict[str, Any]

                if isinstance(results_item_data, SelectFilterListResponse):
                    results_item = results_item_data.to_dict()

                elif isinstance(results_item_data, SelectFilterListDetailedResponse):
                    results_item = results_item_data.to_dict()

                elif isinstance(results_item_data, RangeFilterListResponse):
                    results_item = results_item_data.to_dict()

                else:
                    results_item = results_item_data.to_dict()

                results.append(results_item)

        field_links: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field_links, Unset):
            field_links = self.field_links.to_dict()

        meta: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.meta, Unset):
            meta = self.meta.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if results is not UNSET:
            field_dict["results"] = results
        if field_links is not UNSET:
            field_dict["_links"] = field_links
        if meta is not UNSET:
            field_dict["meta"] = meta

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.range_filter_list_detailed_response import (
            RangeFilterListDetailedResponse,
        )
        from ..models.select_filter_list_detailed_response import (
            SelectFilterListDetailedResponse,
        )
        from ..models.range_filter_list_response import RangeFilterListResponse
        from ..models.filter_list_meta import FilterListMeta
        from ..models.select_filter_list_response import SelectFilterListResponse
        from ..models.filter_list_links import FilterListLinks

        d = src_dict.copy()
        results = []
        _results = d.pop("results", UNSET)
        for results_item_data in _results or []:

            def _parse_results_item(
                data: object,
            ) -> Union[
                "RangeFilterListDetailedResponse",
                "RangeFilterListResponse",
                "SelectFilterListDetailedResponse",
                "SelectFilterListResponse",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    results_item_type_0 = SelectFilterListResponse.from_dict(data)

                    return results_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    results_item_type_1 = SelectFilterListDetailedResponse.from_dict(
                        data
                    )

                    return results_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    results_item_type_2 = RangeFilterListResponse.from_dict(data)

                    return results_item_type_2
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                results_item_type_3 = RangeFilterListDetailedResponse.from_dict(data)

                return results_item_type_3

            results_item = _parse_results_item(results_item_data)

            results.append(results_item)

        _field_links = d.pop("_links", UNSET)
        field_links: Union[Unset, FilterListLinks]
        if isinstance(_field_links, Unset):
            field_links = UNSET
        else:
            field_links = FilterListLinks.from_dict(_field_links)

        _meta = d.pop("meta", UNSET)
        meta: Union[Unset, FilterListMeta]
        if isinstance(_meta, Unset):
            meta = UNSET
        else:
            meta = FilterListMeta.from_dict(_meta)

        filter_list = cls(
            results=results,
            field_links=field_links,
            meta=meta,
        )

        filter_list.additional_properties = d
        return filter_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.transition_mutually_exclusive_study_collection_json_body_action import (
    TransitionMutuallyExclusiveStudyCollectionJsonBodyAction,
)
from ..types import UNSET, Unset


T = TypeVar("T", bound="TransitionMutuallyExclusiveStudyCollectionJsonBody")


@_attrs_define
class TransitionMutuallyExclusiveStudyCollectionJsonBody:
    """
    Attributes:
        action (Union[Unset, TransitionMutuallyExclusiveStudyCollectionJsonBodyAction]):
        publish_at (Union[Unset, str]): Optional parameter for scheduling publish, indicating the datetime and timezone
            the study collection should be scheduled to be published at
    """

    action: Union[
        Unset, TransitionMutuallyExclusiveStudyCollectionJsonBodyAction
    ] = UNSET
    publish_at: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        action: Union[Unset, str] = UNSET
        if not isinstance(self.action, Unset):
            action = self.action.value

        publish_at = self.publish_at

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if action is not UNSET:
            field_dict["action"] = action
        if publish_at is not UNSET:
            field_dict["publish_at"] = publish_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _action = d.pop("action", UNSET)
        action: Union[Unset, TransitionMutuallyExclusiveStudyCollectionJsonBodyAction]
        if isinstance(_action, Unset):
            action = UNSET
        else:
            action = TransitionMutuallyExclusiveStudyCollectionJsonBodyAction(_action)

        publish_at = d.pop("publish_at", UNSET)

        transition_mutually_exclusive_study_collection_json_body = cls(
            action=action,
            publish_at=publish_at,
        )

        transition_mutually_exclusive_study_collection_json_body.additional_properties = d
        return transition_mutually_exclusive_study_collection_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BaseStudyCompletionCodesItemCodeType(str, Enum):
    COMPLETED = "COMPLETED"
    FAILED_ATTENTION_CHECK = "FAILED_ATTENTION_CHECK"
    FOLLOW_UP_STUDY = "FOLLOW_UP_STUDY"
    GIVE_BONUS = "GIVE_BONUS"
    INCOMPATIBLE_DEVICE = "INCOMPATIBLE_DEVICE"
    NO_CONSENT = "NO_CONSENT"
    OTHER = "OTHER"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from ...models.bulk_bonus import BulkBonus
from typing import Dict
from ...models.create_bonus_payments_json_body import CreateBonusPaymentsJsonBody


def _get_kwargs(
    *,
    json_body: CreateBonusPaymentsJsonBody,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/submissions/bonus-payments/",
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[BulkBonus]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = BulkBonus.from_dict(response.json())

        return response_201
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[BulkBonus]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: CreateBonusPaymentsJsonBody,
    authorization: str,
) -> Response[BulkBonus]:
    r"""Set up bonuses

     Set up bonus payments to one or more participants/submissions in a study.

    You need the study id, the participant|submission ids and the request in CSV format.

    The csv_bonuses field needs to be structured in the format of either:

    - `<participant_id>,<amount>\n`.
    - `<submission_id>,<amount>\n`.

    Setting up a bonus payment does not actually pay them, to do so check [/api/v1/bulk-bonus-
    payments/{id}/pay/](#tag/Bonuses/paths/~1api~1v1~1bulk-bonus-payments~1%7Bid%7D~1pay~1/post)

    The submission and participant IDs need to have taken part in the study in order for this call to be
    successful.
    If not, you will get a \"400\" HTTP response back, which explains which IDs are incorrect.

    Args:
        authorization (str):
        json_body (CreateBonusPaymentsJsonBody):  Example: {'study_id':
            '60f6acb180a7b59ac0621f9e', 'csv_bonuses':
            '60ffe5c8371090c7041d43f8,4.25\n60ff44a1d00991f1dfe405d9,4.25'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[BulkBonus]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: CreateBonusPaymentsJsonBody,
    authorization: str,
) -> Optional[BulkBonus]:
    r"""Set up bonuses

     Set up bonus payments to one or more participants/submissions in a study.

    You need the study id, the participant|submission ids and the request in CSV format.

    The csv_bonuses field needs to be structured in the format of either:

    - `<participant_id>,<amount>\n`.
    - `<submission_id>,<amount>\n`.

    Setting up a bonus payment does not actually pay them, to do so check [/api/v1/bulk-bonus-
    payments/{id}/pay/](#tag/Bonuses/paths/~1api~1v1~1bulk-bonus-payments~1%7Bid%7D~1pay~1/post)

    The submission and participant IDs need to have taken part in the study in order for this call to be
    successful.
    If not, you will get a \"400\" HTTP response back, which explains which IDs are incorrect.

    Args:
        authorization (str):
        json_body (CreateBonusPaymentsJsonBody):  Example: {'study_id':
            '60f6acb180a7b59ac0621f9e', 'csv_bonuses':
            '60ffe5c8371090c7041d43f8,4.25\n60ff44a1d00991f1dfe405d9,4.25'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        BulkBonus
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: CreateBonusPaymentsJsonBody,
    authorization: str,
) -> Response[BulkBonus]:
    r"""Set up bonuses

     Set up bonus payments to one or more participants/submissions in a study.

    You need the study id, the participant|submission ids and the request in CSV format.

    The csv_bonuses field needs to be structured in the format of either:

    - `<participant_id>,<amount>\n`.
    - `<submission_id>,<amount>\n`.

    Setting up a bonus payment does not actually pay them, to do so check [/api/v1/bulk-bonus-
    payments/{id}/pay/](#tag/Bonuses/paths/~1api~1v1~1bulk-bonus-payments~1%7Bid%7D~1pay~1/post)

    The submission and participant IDs need to have taken part in the study in order for this call to be
    successful.
    If not, you will get a \"400\" HTTP response back, which explains which IDs are incorrect.

    Args:
        authorization (str):
        json_body (CreateBonusPaymentsJsonBody):  Example: {'study_id':
            '60f6acb180a7b59ac0621f9e', 'csv_bonuses':
            '60ffe5c8371090c7041d43f8,4.25\n60ff44a1d00991f1dfe405d9,4.25'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[BulkBonus]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: CreateBonusPaymentsJsonBody,
    authorization: str,
) -> Optional[BulkBonus]:
    r"""Set up bonuses

     Set up bonus payments to one or more participants/submissions in a study.

    You need the study id, the participant|submission ids and the request in CSV format.

    The csv_bonuses field needs to be structured in the format of either:

    - `<participant_id>,<amount>\n`.
    - `<submission_id>,<amount>\n`.

    Setting up a bonus payment does not actually pay them, to do so check [/api/v1/bulk-bonus-
    payments/{id}/pay/](#tag/Bonuses/paths/~1api~1v1~1bulk-bonus-payments~1%7Bid%7D~1pay~1/post)

    The submission and participant IDs need to have taken part in the study in order for this call to be
    successful.
    If not, you will get a \"400\" HTTP response back, which explains which IDs are incorrect.

    Args:
        authorization (str):
        json_body (CreateBonusPaymentsJsonBody):  Example: {'study_id':
            '60f6acb180a7b59ac0621f9e', 'csv_bonuses':
            '60ffe5c8371090c7041d43f8,4.25\n60ff44a1d00991f1dfe405d9,4.25'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        BulkBonus
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar


from attrs import define as _attrs_define

from ..types import UNSET, Unset

from typing import cast, List
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="MutuallyExclusiveStudyCollectionUpdate")


@_attrs_define
class MutuallyExclusiveStudyCollectionUpdate:
    """
    Attributes:
        name (Union[Unset, str]): Mutually exclusive study collection name Example: My Mutually Exclusive Study
            Collection.
        description (Union[Unset, str]): A description of the study collection Example: This is a description of my
            mutually exclusive study collection.
        publish_at (Union[Unset, None, str]): Datetime and timezone the study collection should be scheduled to be
            published at Example: 2050-02-28T13:45:00 Europe/London.
        study_ids (Union[Unset, List[str]]): List of study ids you wish to include in the collection. Note, this will
            overwrite the current list of studies in the collection
    """

    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    publish_at: Union[Unset, None, str] = UNSET
    study_ids: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        description = self.description
        publish_at = self.publish_at
        study_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.study_ids, Unset):
            study_ids = self.study_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if publish_at is not UNSET:
            field_dict["publish_at"] = publish_at
        if study_ids is not UNSET:
            field_dict["study_ids"] = study_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        publish_at = d.pop("publish_at", UNSET)

        study_ids = cast(List[str], d.pop("study_ids", UNSET))

        mutually_exclusive_study_collection_update = cls(
            name=name,
            description=description,
            publish_at=publish_at,
            study_ids=study_ids,
        )

        return mutually_exclusive_study_collection_update



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.workspace_user import WorkspaceUser
    from ..models.project_short import ProjectShort


T = TypeVar("T", bound="Workspace")


@_attrs_define
class Workspace:
    """
    Example:
        {'id': '62fce6fff0a78eb4f3ebc09c', 'title': 'My workspace', 'description': 'This workspace does...', 'owner':
            '60a42f4c693c29420793cb73', 'users': [{'id': '60a42f4c693c29420793cb73', 'name': 'Joe Soap', 'email':
            'joe.soap@gmail.com', 'roles': ['WORKSPACE_ADMIN']}], 'projects': [{'id': '60a42f4c693c29420793cb73'}],
            'wallet': '61a65c06b084910b3f0c00d6'}

    Attributes:
        id (str): Workspace id. It is created by Prolific.
        title (str): Name of workspace
        description (Union[Unset, str]): What is this workspace used for
        owner (Union[Unset, str]): Workspace id. It is created by Prolific.
        users (Union[Unset, List['WorkspaceUser']]): Data for a user related to a workspace
        projects (Union[Unset, List['ProjectShort']]): Data for a project related to a workspace
        wallet (Union[Unset, str]): Wallet tied to workspace
        naivety_distribution_rate (Union[Unset, None, float]): The rate at which the studies within this workspace are
            distributed.
    """

    id: str
    title: str
    description: Union[Unset, str] = UNSET
    owner: Union[Unset, str] = UNSET
    users: Union[Unset, List["WorkspaceUser"]] = UNSET
    projects: Union[Unset, List["ProjectShort"]] = UNSET
    wallet: Union[Unset, str] = UNSET
    naivety_distribution_rate: Union[Unset, None, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        title = self.title
        description = self.description
        owner = self.owner
        users: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.users, Unset):
            users = []
            for users_item_data in self.users:
                users_item = users_item_data.to_dict()

                users.append(users_item)

        projects: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.projects, Unset):
            projects = []
            for projects_item_data in self.projects:
                projects_item = projects_item_data.to_dict()

                projects.append(projects_item)

        wallet = self.wallet
        naivety_distribution_rate = self.naivety_distribution_rate

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "title": title,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if owner is not UNSET:
            field_dict["owner"] = owner
        if users is not UNSET:
            field_dict["users"] = users
        if projects is not UNSET:
            field_dict["projects"] = projects
        if wallet is not UNSET:
            field_dict["wallet"] = wallet
        if naivety_distribution_rate is not UNSET:
            field_dict["naivety_distribution_rate"] = naivety_distribution_rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workspace_user import WorkspaceUser
        from ..models.project_short import ProjectShort

        d = src_dict.copy()
        id = d.pop("id")

        title = d.pop("title")

        description = d.pop("description", UNSET)

        owner = d.pop("owner", UNSET)

        users = []
        _users = d.pop("users", UNSET)
        for users_item_data in _users or []:
            users_item = WorkspaceUser.from_dict(users_item_data)

            users.append(users_item)

        projects = []
        _projects = d.pop("projects", UNSET)
        for projects_item_data in _projects or []:
            projects_item = ProjectShort.from_dict(projects_item_data)

            projects.append(projects_item)

        wallet = d.pop("wallet", UNSET)

        naivety_distribution_rate = d.pop("naivety_distribution_rate", UNSET)

        workspace = cls(
            id=id,
            title=title,
            description=description,
            owner=owner,
            users=users,
            projects=projects,
            wallet=wallet,
            naivety_distribution_rate=naivety_distribution_rate,
        )

        workspace.additional_properties = d
        return workspace

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import Union
from ...models.project_id import ProjectID
from typing import Union
from ...models.get_participant_groups_active import GetParticipantGroupsActive
from typing import Optional
from ...models.workspace_id import WorkspaceID
from typing import Dict
from ...models.participant_group_list_response import ParticipantGroupListResponse


def _get_kwargs(
    *,
    active: Union[Unset, None, GetParticipantGroupsActive] = UNSET,
    filter_: Union["ProjectID", "WorkspaceID"],
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}
    json_active: Union[Unset, None, str] = UNSET
    if not isinstance(active, Unset):
        json_active = active.value if active else None

    params["active"] = json_active

    json_filter_: Dict[str, Any]

    if isinstance(filter_, WorkspaceID):
        json_filter_ = filter_.to_dict()

    else:
        json_filter_ = filter_.to_dict()

    params["filter"] = json_filter_

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/api/v1/participant-groups/",
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[ParticipantGroupListResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ParticipantGroupListResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[ParticipantGroupListResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    active: Union[Unset, None, GetParticipantGroupsActive] = UNSET,
    filter_: Union["ProjectID", "WorkspaceID"],
) -> Response[ParticipantGroupListResponse]:
    """Get a list of all participant groups within a project or workspace

    Args:
        active (Union[Unset, None, GetParticipantGroupsActive]):
        filter_ (Union['ProjectID', 'WorkspaceID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ParticipantGroupListResponse]
    """

    kwargs = _get_kwargs(
        active=active,
        filter_=filter_,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    active: Union[Unset, None, GetParticipantGroupsActive] = UNSET,
    filter_: Union["ProjectID", "WorkspaceID"],
) -> Optional[ParticipantGroupListResponse]:
    """Get a list of all participant groups within a project or workspace

    Args:
        active (Union[Unset, None, GetParticipantGroupsActive]):
        filter_ (Union['ProjectID', 'WorkspaceID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ParticipantGroupListResponse
    """

    return sync_detailed(
        client=client,
        active=active,
        filter_=filter_,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    active: Union[Unset, None, GetParticipantGroupsActive] = UNSET,
    filter_: Union["ProjectID", "WorkspaceID"],
) -> Response[ParticipantGroupListResponse]:
    """Get a list of all participant groups within a project or workspace

    Args:
        active (Union[Unset, None, GetParticipantGroupsActive]):
        filter_ (Union['ProjectID', 'WorkspaceID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ParticipantGroupListResponse]
    """

    kwargs = _get_kwargs(
        active=active,
        filter_=filter_,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    active: Union[Unset, None, GetParticipantGroupsActive] = UNSET,
    filter_: Union["ProjectID", "WorkspaceID"],
) -> Optional[ParticipantGroupListResponse]:
    """Get a list of all participant groups within a project or workspace

    Args:
        active (Union[Unset, None, GetParticipantGroupsActive]):
        filter_ (Union['ProjectID', 'WorkspaceID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ParticipantGroupListResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            active=active,
            filter_=filter_,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.select_filter_list_attributes_data_type import (
    SelectFilterListAttributesDataType,
)
from ..models.filter_list_attributes_type import FilterListAttributesType
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.select_filter_list_attributes_choices import (
        SelectFilterListAttributesChoices,
    )


T = TypeVar("T", bound="SelectFilterListResponse")


@_attrs_define
class SelectFilterListResponse:
    """
    Attributes:
        filter_id (Union[Unset, str]): The ID of the filter, based on a slugified version of the title at the time the
            filter was created.
        title (Union[Unset, str]): The title of the filter.
        description (Union[Unset, str]): A description of the filter.
        type (Union[Unset, FilterListAttributesType]): The filter type.
        question (Union[Unset, str]): The question asked of participants to generate this filter.
        choices (Union[Unset, SelectFilterListAttributesChoices]): An object containing all the filter's possible
            responses as key-value pairs, with sequential integer IDs or database ObjectIDs as the keys and the text of the
            response as the values.
        data_type (Union[Unset, SelectFilterListAttributesDataType]): The format of the keys in the choices object. If
            the keys are strings representing sequential integers,
            the data format is integer. If the keys are database ObjectIDs, the type of ID is specified.
    """

    filter_id: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    type: Union[Unset, FilterListAttributesType] = UNSET
    question: Union[Unset, str] = UNSET
    choices: Union[Unset, "SelectFilterListAttributesChoices"] = UNSET
    data_type: Union[Unset, SelectFilterListAttributesDataType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        filter_id = self.filter_id
        title = self.title
        description = self.description
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        question = self.question
        choices: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.choices, Unset):
            choices = self.choices.to_dict()

        data_type: Union[Unset, str] = UNSET
        if not isinstance(self.data_type, Unset):
            data_type = self.data_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filter_id is not UNSET:
            field_dict["filter_id"] = filter_id
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if type is not UNSET:
            field_dict["type"] = type
        if question is not UNSET:
            field_dict["question"] = question
        if choices is not UNSET:
            field_dict["choices"] = choices
        if data_type is not UNSET:
            field_dict["data_type"] = data_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.select_filter_list_attributes_choices import (
            SelectFilterListAttributesChoices,
        )

        d = src_dict.copy()
        filter_id = d.pop("filter_id", UNSET)

        title = d.pop("title", UNSET)

        description = d.pop("description", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, FilterListAttributesType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = FilterListAttributesType(_type)

        question = d.pop("question", UNSET)

        _choices = d.pop("choices", UNSET)
        choices: Union[Unset, SelectFilterListAttributesChoices]
        if isinstance(_choices, Unset):
            choices = UNSET
        else:
            choices = SelectFilterListAttributesChoices.from_dict(_choices)

        _data_type = d.pop("data_type", UNSET)
        data_type: Union[Unset, SelectFilterListAttributesDataType]
        if isinstance(_data_type, Unset):
            data_type = UNSET
        else:
            data_type = SelectFilterListAttributesDataType(_data_type)

        select_filter_list_response = cls(
            filter_id=filter_id,
            title=title,
            description=description,
            type=type,
            question=question,
            choices=choices,
            data_type=data_type,
        )

        select_filter_list_response.additional_properties = d
        return select_filter_list_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from typing import Dict
from ...models.create_study import CreateStudy
from ...models.study import Study


def _get_kwargs(
    *,
    json_body: CreateStudy,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/studies/",
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Study]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = Study.from_dict(response.json())

        return response_201
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Study]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: CreateStudy,
    authorization: str,
) -> Response[Study]:
    """Create a draft study

     Create a draft study. Publishing a study is a two step process, first create a draft study then
    publish it.

    Args:
        authorization (str):
        json_body (CreateStudy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Study]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: CreateStudy,
    authorization: str,
) -> Optional[Study]:
    """Create a draft study

     Create a draft study. Publishing a study is a two step process, first create a draft study then
    publish it.

    Args:
        authorization (str):
        json_body (CreateStudy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Study
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: CreateStudy,
    authorization: str,
) -> Response[Study]:
    """Create a draft study

     Create a draft study. Publishing a study is a two step process, first create a draft study then
    publish it.

    Args:
        authorization (str):
        json_body (CreateStudy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Study]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: CreateStudy,
    authorization: str,
) -> Optional[Study]:
    """Create a draft study

     Create a draft study. Publishing a study is a two step process, first create a draft study then
    publish it.

    Args:
        authorization (str):
        json_body (CreateStudy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Study
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.range_filter import RangeFilter
    from ..models.select_filter import SelectFilter


T = TypeVar("T", bound="RequirementsCountRequest")


@_attrs_define
class RequirementsCountRequest:
    """
    Attributes:
        filters (List[Union['RangeFilter', 'SelectFilter']]): List of filters to apply to the count. This parameter uses
            the new, simplified
            filters schema for interacting with eligibility.
        workspace_id (Union[Unset, str]): The ID of the workspace you will be creating a study in.

            Due to US tax laws, non US residents may not participate in studies created by US researchers.
            For this reason, we use the country specified in the workspace to determine eligibility.

            If you do not specify a workspace ID, we will use the current workspace ID of the user making the request.
            Your eligibility count may not be accurate if you do not specify a workspace ID.
        organisation_id (Union[Unset, str]): The ID of the workspace you will be creating a filterset in.
    """

    filters: List[Union["RangeFilter", "SelectFilter"]]
    workspace_id: Union[Unset, str] = UNSET
    organisation_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.select_filter import SelectFilter

        filters = []
        for filters_item_data in self.filters:
            filters_item: Dict[str, Any]

            if isinstance(filters_item_data, SelectFilter):
                filters_item = filters_item_data.to_dict()

            else:
                filters_item = filters_item_data.to_dict()

            filters.append(filters_item)

        workspace_id = self.workspace_id
        organisation_id = self.organisation_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "filters": filters,
            }
        )
        if workspace_id is not UNSET:
            field_dict["workspace_id"] = workspace_id
        if organisation_id is not UNSET:
            field_dict["organisation_id"] = organisation_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.range_filter import RangeFilter
        from ..models.select_filter import SelectFilter

        d = src_dict.copy()
        filters = []
        _filters = d.pop("filters")
        for filters_item_data in _filters:

            def _parse_filters_item(
                data: object,
            ) -> Union["RangeFilter", "SelectFilter"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_0 = SelectFilter.from_dict(data)

                    return filters_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                filters_item_type_1 = RangeFilter.from_dict(data)

                return filters_item_type_1

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        workspace_id = d.pop("workspace_id", UNSET)

        organisation_id = d.pop("organisation_id", UNSET)

        requirements_count_request = cls(
            filters=filters,
            workspace_id=workspace_id,
            organisation_id=organisation_id,
        )

        requirements_count_request.additional_properties = d
        return requirements_count_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.attribute import Attribute
    from ..models.requirement_query import RequirementQuery


T = TypeVar("T", bound="Requirement")


@_attrs_define
class Requirement:
    """
    Attributes:
        field_cls (str): Type of requirement Example: SelectAnswerEligibilityRequirement.
        attributes (List['Attribute']): Attributes defining the requirement. Its values will depend
            on the type of requirement
        query (Union[Unset, RequirementQuery]):
    """

    field_cls: str
    attributes: List["Attribute"]
    query: Union[Unset, "RequirementQuery"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_cls = self.field_cls
        attributes = []
        for attributes_item_data in self.attributes:
            attributes_item = attributes_item_data.to_dict()

            attributes.append(attributes_item)

        query: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query, Unset):
            query = self.query.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "_cls": field_cls,
                "attributes": attributes,
            }
        )
        if query is not UNSET:
            field_dict["query"] = query

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.attribute import Attribute
        from ..models.requirement_query import RequirementQuery

        d = src_dict.copy()
        field_cls = d.pop("_cls")

        attributes = []
        _attributes = d.pop("attributes")
        for attributes_item_data in _attributes:
            attributes_item = Attribute.from_dict(attributes_item_data)

            attributes.append(attributes_item)

        _query = d.pop("query", UNSET)
        query: Union[Unset, RequirementQuery]
        if isinstance(_query, Unset):
            query = UNSET
        else:
            query = RequirementQuery.from_dict(_query)

        requirement = cls(
            field_cls=field_cls,
            attributes=attributes,
            query=query,
        )

        requirement.additional_properties = d
        return requirement

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.create_invitation_request_role import CreateInvitationRequestRole
from typing import cast, List
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="CreateInvitationRequest")


@_attrs_define
class CreateInvitationRequest:
    """
    Attributes:
        association (Union[Unset, str]): The ID of the workspace to which the users are being invited.
        emails (Union[Unset, List[str]]): An array of email addresses of the users to invite.
        role (Union[Unset, CreateInvitationRequestRole]): The role that the invited users will have in the workspace or
            project. This can be one of the following:
            - "WORKSPACE_ADMIN": The user will have administrative rights in the workspace. They can manage settings, invite
            users, and oversee all projects.
            - "WORKSPACE_COLLABORATOR": The user will be a regular collaborator in the workspace. They can contribute to
            projects but don't have administrative rights.
            - "PROJECT_EDITOR": The user will have edit rights for a specific project within a workspace.
    """

    association: Union[Unset, str] = UNSET
    emails: Union[Unset, List[str]] = UNSET
    role: Union[Unset, CreateInvitationRequestRole] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        association = self.association
        emails: Union[Unset, List[str]] = UNSET
        if not isinstance(self.emails, Unset):
            emails = self.emails

        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if association is not UNSET:
            field_dict["association"] = association
        if emails is not UNSET:
            field_dict["emails"] = emails
        if role is not UNSET:
            field_dict["role"] = role

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        association = d.pop("association", UNSET)

        emails = cast(List[str], d.pop("emails", UNSET))

        _role = d.pop("role", UNSET)
        role: Union[Unset, CreateInvitationRequestRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = CreateInvitationRequestRole(_role)

        create_invitation_request = cls(
            association=association,
            emails=emails,
            role=role,
        )

        create_invitation_request.additional_properties = d
        return create_invitation_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import List
from typing import Dict

if TYPE_CHECKING:
    from ..models.requirement import Requirement


T = TypeVar("T", bound="RequirementsResponse")


@_attrs_define
class RequirementsResponse:
    """
    Attributes:
        results (List['Requirement']): List of all defined requirements. There are many attributes
            per requirement that are not documented and are used by our webapp.
    """

    results: List["Requirement"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        results = []
        for results_item_data in self.results:
            results_item = results_item_data.to_dict()

            results.append(results_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "results": results,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.requirement import Requirement

        d = src_dict.copy()
        results = []
        _results = d.pop("results")
        for results_item_data in _results:
            results_item = Requirement.from_dict(results_item_data)

            results.append(results_item)

        requirements_response = cls(
            results=results,
        )

        requirements_response.additional_properties = d
        return requirements_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="CreateBonusPaymentsJsonBody")


@_attrs_define
class CreateBonusPaymentsJsonBody:
    r"""
    Example:
        {'study_id': '60f6acb180a7b59ac0621f9e', 'csv_bonuses':
            '60ffe5c8371090c7041d43f8,4.25\n60ff44a1d00991f1dfe405d9,4.25'}

    Attributes:
        study_id (Union[Unset, str]):
        csv_bonuses (Union[Unset, str]):
    """

    study_id: Union[Unset, str] = UNSET
    csv_bonuses: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        study_id = self.study_id
        csv_bonuses = self.csv_bonuses

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if study_id is not UNSET:
            field_dict["study_id"] = study_id
        if csv_bonuses is not UNSET:
            field_dict["csv_bonuses"] = csv_bonuses

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        study_id = d.pop("study_id", UNSET)

        csv_bonuses = d.pop("csv_bonuses", UNSET)

        create_bonus_payments_json_body = cls(
            study_id=study_id,
            csv_bonuses=csv_bonuses,
        )

        create_bonus_payments_json_body.additional_properties = d
        return create_bonus_payments_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
import datetime
from ..models.submission_short_status import SubmissionShortStatus
from ..types import UNSET, Unset
from dateutil.parser import isoparse


T = TypeVar("T", bound="SubmissionShort")


@_attrs_define
class SubmissionShort:
    """
    Example:
        {'id': '60d9aadeb86739de712faee0', 'participant_id': '60bf9310e8dec401be6e9615', 'started_at':
            datetime.datetime(2021, 5, 20, 11, 3, 0, 457000, tzinfo=datetime.timezone.utc), 'status': 'ACTIVE',
            'study_code': 'ABC123'}

    Attributes:
        id (str): Submission id.
        participant_id (str): Participant id.
        status (SubmissionShortStatus): Status of the submission.
        started_at (datetime.datetime): Date started
        has_siblings (bool): Whether or not the submission has sibling submissions (sharing the same study).
        completed_at (Union[Unset, None, datetime.datetime]): Date completed
        study_code (Union[Unset, None, str]): The completion code used by the participant to complete the study.
    """

    id: str
    participant_id: str
    status: SubmissionShortStatus
    started_at: datetime.datetime
    has_siblings: bool
    completed_at: Union[Unset, None, datetime.datetime] = UNSET
    study_code: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        participant_id = self.participant_id
        status = self.status.value

        started_at = self.started_at.isoformat()

        has_siblings = self.has_siblings
        completed_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat() if self.completed_at else None

        study_code = self.study_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "participant_id": participant_id,
                "status": status,
                "started_at": started_at,
                "has_siblings": has_siblings,
            }
        )
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if study_code is not UNSET:
            field_dict["study_code"] = study_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        participant_id = d.pop("participant_id")

        status = SubmissionShortStatus(d.pop("status"))

        started_at = isoparse(d.pop("started_at"))

        has_siblings = d.pop("has_siblings")

        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, None, datetime.datetime]
        if _completed_at is None:
            completed_at = None
        elif isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        study_code = d.pop("study_code", UNSET)

        submission_short = cls(
            id=id,
            participant_id=participant_id,
            status=status,
            started_at=started_at,
            has_siblings=has_siblings,
            completed_at=completed_at,
            study_code=study_code,
        )

        submission_short.additional_properties = d
        return submission_short

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import Union
from ...models.studies_list_response import StudiesListResponse
from ...models.get_studies_state import GetStudiesState
from typing import Optional
from typing import Dict


def _get_kwargs(
    *,
    state: Union[Unset, None, GetStudiesState] = UNSET,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    params: Dict[str, Any] = {}
    json_state: Union[Unset, None, str] = UNSET
    if not isinstance(state, Unset):
        json_state = state.value if state else None

    params["state"] = json_state

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/api/v1/studies/",
        "params": params,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[StudiesListResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = StudiesListResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[StudiesListResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    state: Union[Unset, None, GetStudiesState] = UNSET,
    authorization: str,
) -> Response[StudiesListResponse]:
    """List all studies

     List all studies, with the option to filter by study status.

    Args:
        state (Union[Unset, None, GetStudiesState]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[StudiesListResponse]
    """

    kwargs = _get_kwargs(
        state=state,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    state: Union[Unset, None, GetStudiesState] = UNSET,
    authorization: str,
) -> Optional[StudiesListResponse]:
    """List all studies

     List all studies, with the option to filter by study status.

    Args:
        state (Union[Unset, None, GetStudiesState]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        StudiesListResponse
    """

    return sync_detailed(
        client=client,
        state=state,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    state: Union[Unset, None, GetStudiesState] = UNSET,
    authorization: str,
) -> Response[StudiesListResponse]:
    """List all studies

     List all studies, with the option to filter by study status.

    Args:
        state (Union[Unset, None, GetStudiesState]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[StudiesListResponse]
    """

    kwargs = _get_kwargs(
        state=state,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    state: Union[Unset, None, GetStudiesState] = UNSET,
    authorization: str,
) -> Optional[StudiesListResponse]:
    """List all studies

     List all studies, with the option to filter by study status.

    Args:
        state (Union[Unset, None, GetStudiesState]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        StudiesListResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            state=state,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast, Union
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="RangeFilterSelectedRange")


@_attrs_define
class RangeFilterSelectedRange:
    r"""This schema applies for filters of the `range` type, as defined in the [filter list
    response](\#tag/Filters/paths/~1api~1v1~1filters~1/get).

    A dictionary with two possible objects, 'lower' and 'upper'. At least one must be present and a non-null value.

    The expected data type for these values is defined by the `range` filter's `data_type` (see response linked above).

    If the data_type is a date, string format should be a parseable ISO8601 date string. Date values should be provided
    as a string in ISO 8601 format.

    Leaving a value as null will result in that bound being set to the lowest or highest possible value, depending on
    whether it is the upper or lower bound.

        Attributes:
            lower (Union[Unset, int, str]): Your selected lower bound for the range.
            upper (Union[Unset, int, str]): Your selected upper bound for the range.
    """

    lower: Union[Unset, int, str] = UNSET
    upper: Union[Unset, int, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        lower: Union[Unset, int, str]
        if isinstance(self.lower, Unset):
            lower = UNSET

        else:
            lower = self.lower

        upper: Union[Unset, int, str]
        if isinstance(self.upper, Unset):
            upper = UNSET

        else:
            upper = self.upper

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if lower is not UNSET:
            field_dict["lower"] = lower
        if upper is not UNSET:
            field_dict["upper"] = upper

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_lower(data: object) -> Union[Unset, int, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Unset, int, str], data)

        lower = _parse_lower(d.pop("lower", UNSET))

        def _parse_upper(data: object) -> Union[Unset, int, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Unset, int, str], data)

        upper = _parse_upper(d.pop("upper", UNSET))

        range_filter_selected_range = cls(
            lower=lower,
            upper=upper,
        )

        range_filter_selected_range.additional_properties = d
        return range_filter_selected_range

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.summary_answer import SummaryAnswer


T = TypeVar("T", bound="SummaryQuestion")


@_attrs_define
class SummaryQuestion:
    """Responsible for housing the questions we want to aggregate for the summary.

    Attributes:
        question (str): The title of the question.
        question_id (Union[Unset, str]): The question ID.
        total_answers (Union[Unset, int]): The total number of answered responses for a given question.
        answers (Union[Unset, List['SummaryAnswer']]): A list of aggregated answer information.
    """

    question: str
    question_id: Union[Unset, str] = UNSET
    total_answers: Union[Unset, int] = 0
    answers: Union[Unset, List["SummaryAnswer"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        question = self.question
        question_id = self.question_id
        total_answers = self.total_answers
        answers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.answers, Unset):
            answers = []
            for answers_item_data in self.answers:
                answers_item = answers_item_data.to_dict()

                answers.append(answers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "question": question,
            }
        )
        if question_id is not UNSET:
            field_dict["question_id"] = question_id
        if total_answers is not UNSET:
            field_dict["total_answers"] = total_answers
        if answers is not UNSET:
            field_dict["answers"] = answers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.summary_answer import SummaryAnswer

        d = src_dict.copy()
        question = d.pop("question")

        question_id = d.pop("question_id", UNSET)

        total_answers = d.pop("total_answers", UNSET)

        answers = []
        _answers = d.pop("answers", UNSET)
        for answers_item_data in _answers or []:
            answers_item = SummaryAnswer.from_dict(answers_item_data)

            answers.append(answers_item)

        summary_question = cls(
            question=question,
            question_id=question_id,
            total_answers=total_answers,
            answers=answers,
        )

        summary_question.additional_properties = d
        return summary_question

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="CreateWorkspace")


@_attrs_define
class CreateWorkspace:
    """
    Attributes:
        title (str): Name of workspace
        naivety_distribution_rate (Union[Unset, None, float]): Control the balance between speed of your studies and the
            naivety of the participants.

            If not defined, by default Prolific calculates the best rate for most studies
            taking into account the `filters` and the `total_available_places` needed for this study.

            Use 0 if your priority is speed. When this property is set to 0 all eligible participants will have access
            to your study at the same time, without any prioritization.

            You can also set this at a project and study level.
        currency_code (Union[Unset, str]): Currency used for all transactions within the workspace. Must be GBP or USD.
    """

    title: str
    naivety_distribution_rate: Union[Unset, None, float] = UNSET
    currency_code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        naivety_distribution_rate = self.naivety_distribution_rate
        currency_code = self.currency_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "title": title,
            }
        )
        if naivety_distribution_rate is not UNSET:
            field_dict["naivety_distribution_rate"] = naivety_distribution_rate
        if currency_code is not UNSET:
            field_dict["currency_code"] = currency_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title")

        naivety_distribution_rate = d.pop("naivety_distribution_rate", UNSET)

        currency_code = d.pop("currency_code", UNSET)

        create_workspace = cls(
            title=title,
            naivety_distribution_rate=naivety_distribution_rate,
            currency_code=currency_code,
        )

        create_workspace.additional_properties = d
        return create_workspace

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from ...models.messages import Messages
from typing import Dict


def _get_kwargs(
    *,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    return {
        "method": "get",
        "url": "/api/v1/messages/unread/",
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Messages]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Messages.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Messages]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    authorization: str,
) -> Response[Messages]:
    """Retrieve unread messages

     Get all unread messages.
    The messages you have sent are never returned, only messages you have received and not read.
    It does not mark those messages as read.

    Args:
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Messages]
    """

    kwargs = _get_kwargs(
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    authorization: str,
) -> Optional[Messages]:
    """Retrieve unread messages

     Get all unread messages.
    The messages you have sent are never returned, only messages you have received and not read.
    It does not mark those messages as read.

    Args:
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Messages
    """

    return sync_detailed(
        client=client,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    authorization: str,
) -> Response[Messages]:
    """Retrieve unread messages

     Get all unread messages.
    The messages you have sent are never returned, only messages you have received and not read.
    It does not mark those messages as read.

    Args:
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Messages]
    """

    kwargs = _get_kwargs(
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    authorization: str,
) -> Optional[Messages]:
    """Retrieve unread messages

     Get all unread messages.
    The messages you have sent are never returned, only messages you have received and not read.
    It does not mark those messages as read.

    Args:
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Messages
    """

    return (
        await asyncio_detailed(
            client=client,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.range_filter_list_attributes_data_type import (
    RangeFilterListAttributesDataType,
)
from typing import Union
from ..models.filter_list_attributes_type import FilterListAttributesType
from typing import cast, Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="RangeFilterListResponse")


@_attrs_define
class RangeFilterListResponse:
    """
    Attributes:
        filter_id (Union[Unset, str]): The ID of the filter, based on a slugified version of the title at the time the
            filter was created.
        title (Union[Unset, str]): The title of the filter.
        description (Union[Unset, str]): A description of the filter.
        type (Union[Unset, FilterListAttributesType]): The filter type.
        question (Union[Unset, str]): The question asked of participants to generate this filter.
        min_ (Union[Unset, int, str]): The minimum valid value of the range.
        max_ (Union[Unset, int, str]): The maximum valid value of the range.
        data_type (Union[Unset, RangeFilterListAttributesDataType]): The data type of the range. If the data type is
            integer, the lower and upper values must be integers.
            If the data type is date, the lower and upper values must be ISO8601 dates.
    """

    filter_id: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    type: Union[Unset, FilterListAttributesType] = UNSET
    question: Union[Unset, str] = UNSET
    min_: Union[Unset, int, str] = UNSET
    max_: Union[Unset, int, str] = UNSET
    data_type: Union[Unset, RangeFilterListAttributesDataType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        filter_id = self.filter_id
        title = self.title
        description = self.description
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        question = self.question
        min_: Union[Unset, int, str]
        if isinstance(self.min_, Unset):
            min_ = UNSET

        else:
            min_ = self.min_

        max_: Union[Unset, int, str]
        if isinstance(self.max_, Unset):
            max_ = UNSET

        else:
            max_ = self.max_

        data_type: Union[Unset, str] = UNSET
        if not isinstance(self.data_type, Unset):
            data_type = self.data_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filter_id is not UNSET:
            field_dict["filter_id"] = filter_id
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if type is not UNSET:
            field_dict["type"] = type
        if question is not UNSET:
            field_dict["question"] = question
        if min_ is not UNSET:
            field_dict["min"] = min_
        if max_ is not UNSET:
            field_dict["max"] = max_
        if data_type is not UNSET:
            field_dict["data_type"] = data_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        filter_id = d.pop("filter_id", UNSET)

        title = d.pop("title", UNSET)

        description = d.pop("description", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, FilterListAttributesType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = FilterListAttributesType(_type)

        question = d.pop("question", UNSET)

        def _parse_min_(data: object) -> Union[Unset, int, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Unset, int, str], data)

        min_ = _parse_min_(d.pop("min", UNSET))

        def _parse_max_(data: object) -> Union[Unset, int, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Unset, int, str], data)

        max_ = _parse_max_(d.pop("max", UNSET))

        _data_type = d.pop("data_type", UNSET)
        data_type: Union[Unset, RangeFilterListAttributesDataType]
        if isinstance(_data_type, Unset):
            data_type = UNSET
        else:
            data_type = RangeFilterListAttributesDataType(_data_type)

        range_filter_list_response = cls(
            filter_id=filter_id,
            title=title,
            description=description,
            type=type,
            question=question,
            min_=min_,
            max_=max_,
            data_type=data_type,
        )

        range_filter_list_response.additional_properties = d
        return range_filter_list_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from ...models.create_invitation_request import CreateInvitationRequest
from typing import Dict
from ...models.create_invitation_response import CreateInvitationResponse


def _get_kwargs(
    *,
    json_body: CreateInvitationRequest,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/invitations/",
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[CreateInvitationResponse]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = CreateInvitationResponse.from_dict(response.json())

        return response_201
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[CreateInvitationResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: CreateInvitationRequest,
    authorization: str,
) -> Response[CreateInvitationResponse]:
    """Create a new invitation

     Initiate a new invitation process for adding users to a Workspace or Project.

    This operation can be performed only by authenticated users who are admins for the specified
    workspace. Invitations will be sent to the email addresses provided in the request.

    Args:
        authorization (str):
        json_body (CreateInvitationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CreateInvitationResponse]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: CreateInvitationRequest,
    authorization: str,
) -> Optional[CreateInvitationResponse]:
    """Create a new invitation

     Initiate a new invitation process for adding users to a Workspace or Project.

    This operation can be performed only by authenticated users who are admins for the specified
    workspace. Invitations will be sent to the email addresses provided in the request.

    Args:
        authorization (str):
        json_body (CreateInvitationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CreateInvitationResponse
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: CreateInvitationRequest,
    authorization: str,
) -> Response[CreateInvitationResponse]:
    """Create a new invitation

     Initiate a new invitation process for adding users to a Workspace or Project.

    This operation can be performed only by authenticated users who are admins for the specified
    workspace. Invitations will be sent to the email addresses provided in the request.

    Args:
        authorization (str):
        json_body (CreateInvitationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CreateInvitationResponse]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: CreateInvitationRequest,
    authorization: str,
) -> Optional[CreateInvitationResponse]:
    """Create a new invitation

     Initiate a new invitation process for adding users to a Workspace or Project.

    This operation can be performed only by authenticated users who are admins for the specified
    workspace. Invitations will be sent to the email addresses provided in the request.

    Args:
        authorization (str):
        json_body (CreateInvitationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CreateInvitationResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import Union
from typing import List
from ..models.base_study_completion_codes_item_code_type import (
    BaseStudyCompletionCodesItemCodeType,
)
from typing import Dict

if TYPE_CHECKING:
    from ..models.automatically_approve import AutomaticallyApprove
    from ..models.add_to_participant_group import AddToParticipantGroup
    from ..models.manually_review import ManuallyReview
    from ..models.remove_from_participant_group import RemoveFromParticipantGroup
    from ..models.request_return import RequestReturn


T = TypeVar("T", bound="BaseStudyCompletionCodesItem")


@_attrs_define
class BaseStudyCompletionCodesItem:
    """
    Attributes:
        code (str): The code the participant will either enter manually at the end of your study or be redirected as
            part of the return URL.
        code_type (BaseStudyCompletionCodesItemCodeType): A name for your code to make it easier to understand its
            intention. Either use one of the predefined options or any other free text.
        actions (List[Union['AddToParticipantGroup', 'AutomaticallyApprove', 'ManuallyReview',
            'RemoveFromParticipantGroup', 'RequestReturn']]): The actions that will be completed automatically when the
            submission is completed with this code.

            You can specify as many actions as you like. For a basic approach where all submissions are left for manual
            approval, set the `{"action": "MANUALLY_REVIEW"}` option only..
    """

    code: str
    code_type: BaseStudyCompletionCodesItemCodeType
    actions: List[
        Union[
            "AddToParticipantGroup",
            "AutomaticallyApprove",
            "ManuallyReview",
            "RemoveFromParticipantGroup",
            "RequestReturn",
        ]
    ]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.automatically_approve import AutomaticallyApprove
        from ..models.add_to_participant_group import AddToParticipantGroup
        from ..models.manually_review import ManuallyReview
        from ..models.remove_from_participant_group import RemoveFromParticipantGroup

        code = self.code
        code_type = self.code_type.value

        actions = []
        for actions_item_data in self.actions:
            actions_item: Dict[str, Any]

            if isinstance(actions_item_data, AutomaticallyApprove):
                actions_item = actions_item_data.to_dict()

            elif isinstance(actions_item_data, AddToParticipantGroup):
                actions_item = actions_item_data.to_dict()

            elif isinstance(actions_item_data, RemoveFromParticipantGroup):
                actions_item = actions_item_data.to_dict()

            elif isinstance(actions_item_data, ManuallyReview):
                actions_item = actions_item_data.to_dict()

            else:
                actions_item = actions_item_data.to_dict()

            actions.append(actions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "code": code,
                "code_type": code_type,
                "actions": actions,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.automatically_approve import AutomaticallyApprove
        from ..models.add_to_participant_group import AddToParticipantGroup
        from ..models.manually_review import ManuallyReview
        from ..models.remove_from_participant_group import RemoveFromParticipantGroup
        from ..models.request_return import RequestReturn

        d = src_dict.copy()
        code = d.pop("code")

        code_type = BaseStudyCompletionCodesItemCodeType(d.pop("code_type"))

        actions = []
        _actions = d.pop("actions")
        for actions_item_data in _actions:

            def _parse_actions_item(
                data: object,
            ) -> Union[
                "AddToParticipantGroup",
                "AutomaticallyApprove",
                "ManuallyReview",
                "RemoveFromParticipantGroup",
                "RequestReturn",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    actions_item_type_0 = AutomaticallyApprove.from_dict(data)

                    return actions_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    actions_item_type_1 = AddToParticipantGroup.from_dict(data)

                    return actions_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    actions_item_type_2 = RemoveFromParticipantGroup.from_dict(data)

                    return actions_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    actions_item_type_3 = ManuallyReview.from_dict(data)

                    return actions_item_type_3
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                actions_item_type_4 = RequestReturn.from_dict(data)

                return actions_item_type_4

            actions_item = _parse_actions_item(actions_item_data)

            actions.append(actions_item)

        base_study_completion_codes_item = cls(
            code=code,
            code_type=code_type,
            actions=actions,
        )

        base_study_completion_codes_item.additional_properties = d
        return base_study_completion_codes_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CreateInvitationRequestRole(str, Enum):
    PROJECT_EDITOR = "PROJECT_EDITOR"
    WORKSPACE_ADMIN = "WORKSPACE_ADMIN"
    WORKSPACE_COLLABORATOR = "WORKSPACE_COLLABORATOR"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BaseStudyStudyLabelsItem(str, Enum):
    ANNOTATION = "annotation"
    INTERVIEW = "interview"
    OTHER = "other"
    SURVEY = "survey"
    WRITING_TASK = "writing_task"

    def __str__(self) -> str:
        return str(self.value)



""" Contains some shared types for properties """
from http import HTTPStatus
from typing import BinaryIO, Generic, MutableMapping, Optional, Tuple, TypeVar, Literal

from attrs import define


class Unset:
    def __bool__(self) -> Literal[False]:
        return False


UNSET: Unset = Unset()

FileJsonType = Tuple[Optional[str], BinaryIO, Optional[str]]


@define
class File:
    """Contains information for file uploads"""

    payload: BinaryIO
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> FileJsonType:
        """Return a tuple representation that httpx will accept for multipart/form-data"""
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


@define
class Response(Generic[T]):
    """A response from an endpoint"""

    status_code: HTTPStatus
    content: bytes
    headers: MutableMapping[str, str]
    parsed: Optional[T]


__all__ = ["File", "Response", "FileJsonType"]



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.subscription_event_status import SubscriptionEventStatus
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.subscription_event_payload import SubscriptionEventPayload


T = TypeVar("T", bound="SubscriptionEvent")


@_attrs_define
class SubscriptionEvent:
    """
    Attributes:
        id (Union[Unset, str]): The ID of the subscription event.
        datetime_created (Union[Unset, str]): The time the event was created.
        datetime_updated (Union[Unset, str]): The last time the event was updated.
        event_type (Union[Unset, str]): The event type that was triggered.
        resource_id (Union[Unset, str]): The Prolific Resource ID that the event is linked to.
        status (Union[Unset, SubscriptionEventStatus]): The status of the event. Will be `FAILED` if the `target_url`
            response is not 2xx.
        target_url (Union[Unset, str]): The URL where the event payload is sent.
        payload (Union[Unset, None, SubscriptionEventPayload]): The event payload that was sent to the target url.
    """

    id: Union[Unset, str] = UNSET
    datetime_created: Union[Unset, str] = UNSET
    datetime_updated: Union[Unset, str] = UNSET
    event_type: Union[Unset, str] = UNSET
    resource_id: Union[Unset, str] = UNSET
    status: Union[Unset, SubscriptionEventStatus] = UNSET
    target_url: Union[Unset, str] = UNSET
    payload: Union[Unset, None, "SubscriptionEventPayload"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        datetime_created = self.datetime_created
        datetime_updated = self.datetime_updated
        event_type = self.event_type
        resource_id = self.resource_id
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        target_url = self.target_url
        payload: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.payload, Unset):
            payload = self.payload.to_dict() if self.payload else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if datetime_created is not UNSET:
            field_dict["datetime_created"] = datetime_created
        if datetime_updated is not UNSET:
            field_dict["datetime_updated"] = datetime_updated
        if event_type is not UNSET:
            field_dict["event_type"] = event_type
        if resource_id is not UNSET:
            field_dict["resource_id"] = resource_id
        if status is not UNSET:
            field_dict["status"] = status
        if target_url is not UNSET:
            field_dict["target_url"] = target_url
        if payload is not UNSET:
            field_dict["payload"] = payload

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.subscription_event_payload import SubscriptionEventPayload

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        datetime_created = d.pop("datetime_created", UNSET)

        datetime_updated = d.pop("datetime_updated", UNSET)

        event_type = d.pop("event_type", UNSET)

        resource_id = d.pop("resource_id", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, SubscriptionEventStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = SubscriptionEventStatus(_status)

        target_url = d.pop("target_url", UNSET)

        _payload = d.pop("payload", UNSET)
        payload: Union[Unset, None, SubscriptionEventPayload]
        if _payload is None:
            payload = None
        elif isinstance(_payload, Unset):
            payload = UNSET
        else:
            payload = SubscriptionEventPayload.from_dict(_payload)

        subscription_event = cls(
            id=id,
            datetime_created=datetime_created,
            datetime_updated=datetime_updated,
            event_type=event_type,
            resource_id=resource_id,
            status=status,
            target_url=target_url,
            payload=payload,
        )

        subscription_event.additional_properties = d
        return subscription_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
